/**
 * Copyright (c) 2023 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Adam Midlik <midlik@gmail.com>
 *
 * Example command-line application generating images of PDB structures
 * Build: npm install --no-save gl jpeg-js pngjs  // these packages are not listed in dependencies for performance reasons
 *        npm run build
 * Run:   node lib/commonjs/examples/image-renderer 1cbs ../outputs_1cbs/
 */

import { ArgumentParser } from 'argparse';
import fs from 'fs';
import path from 'path';
import gl from 'gl';
import pngjs from 'pngjs';
import jpegjs from 'jpeg-js';

import { HeadlessPluginContext } from '../mol-plugin/headless-plugin-context';
import { DefaultPluginSpec } from '../mol-plugin/spec';
import { ExternalModules, STYLIZED_POSTPROCESSING } from '../mol-plugin/util/headless-screenshot';
import { setFSModule } from '../mol-util/data-source';
import { MembraneOrientationPreset, loadWithUNITMPMembraneRepresentation } from '../tmdet-extension/behavior';
import { TmDetColorThemeProvider } from '../tmdet-extension/tmdet-color-theme';
import { TmDetLabelProvider } from '../tmdet-extension/labeling';


setFSModule(fs);

interface Args {
    pdbId: string,
    outDirectory: string
}

function parseArguments(): Args {
    const parser = new ArgumentParser({ description: 'Example command-line application generating images of PDB structures' });
    parser.add_argument('pdbId', { help: 'PDB identifier' });
    parser.add_argument('outDirectory', { help: 'Directory for outputs' });
    const args = parser.parse_args();
    return { ...args };
}

async function main() {
    const args = parseArguments();
    const url = `https://pdbtm.unitmp.org/api/v1/entry/${args.pdbId}.updated.cif`;
    console.log('PDB ID:', args.pdbId);
    console.log('Source URL:', url);
    console.log('Outputs:', args.outDirectory);

    // Create a headless plugin
    const externalModules: ExternalModules = { gl, pngjs, 'jpeg-js': jpegjs };
    const plugin = new HeadlessPluginContext(externalModules, DefaultPluginSpec(), { width: 800, height: 800 });
    await plugin.init();



    //
    // TMDET extension registration section
    //
    let tree = plugin.state.behaviors.build();

    plugin.builders.structure.representation.registerPreset(MembraneOrientationPreset);
    plugin.representation.structure.themes.colorThemeRegistry.add(TmDetColorThemeProvider);
    plugin.managers.lociLabels.addProvider(TmDetLabelProvider);

    await plugin.runTask(plugin.state.behaviors.updateTree(tree, { doNotUpdateCurrent: true, doNotLogTiming: true }));
    await loadWithUNITMPMembraneRepresentation(plugin, {
        structureUrl: "https://pdbtm.unitmp.org/api/v1/entry/1afo.updated.cif",
        regionDescriptorUrl: "https://pdbtm.unitmp.org/api/v1/entry/1afo.json",
        side1: "Outside"
    });
    //
    // End of TMDET extension registration section
    //



    // Export images
    fs.mkdirSync(args.outDirectory, { recursive: true });
    setTimeout(async () => {
            plugin.saveImage(path.join(args.outDirectory, 'stylized.jpg'), undefined, STYLIZED_POSTPROCESSING);
            // Cleanup
            await plugin.clear();
            plugin.dispose();
        },
        5000
    );

}

main();
